@@ -1,17 +1,20 @@
-from __future__ import division, absolute_import, print_function
-
 import os
 import sys
+import textwrap
 import types
 import re
+import warnings
 
 from numpy.core.numerictypes import issubclass_, issubsctype, issubdtype
-from numpy.core import product, ndarray, ufunc, asarray
+from numpy.core.overrides import set_module
+from numpy.core import ndarray, ufunc, asarray
+import numpy as np
 
-__all__ = ['issubclass_', 'issubsctype', 'issubdtype',
-        'deprecate', 'deprecate_with_doc', 'get_numarray_include',
-        'get_include', 'info', 'source', 'who', 'lookfor', 'byte_bounds',
-        'safe_eval']
+__all__ = [
+    'issubclass_', 'issubsctype', 'issubdtype', 'deprecate',
+    'deprecate_with_doc', 'get_include', 'info', 'source', 'who',
+    'lookfor', 'byte_bounds', 'safe_eval'
+    ]
 
 def get_include():
     """
@@ -22,8 +25,7 @@ def get_include():
 
     Notes
     -----
-    When using ``distutils``, for example in ``setup.py``.
-    ::
+    When using ``distutils``, for example in ``setup.py``::
 
         import numpy as np
         ...
@@ -42,54 +44,13 @@ def get_include():
         d = os.path.join(os.path.dirname(core.__file__), 'include')
     return d
 
-def get_numarray_include(type=None):
-    """
-    Return the directory that contains the numarray \\*.h header files.
-
-    Extension modules that need to compile against numarray should use this
-    function to locate the appropriate include directory.
-
-    Parameters
-    ----------
-    type : any, optional
-        If `type` is not None, the location of the NumPy headers is returned
-        as well.
-
-    Returns
-    -------
-    dirs : str or list of str
-        If `type` is None, `dirs` is a string containing the path to the
-        numarray headers.
-        If `type` is not None, `dirs` is a list of strings with first the
-        path(s) to the numarray headers, followed by the path to the NumPy
-        headers.
-
-    Notes
-    -----
-    Useful when using ``distutils``, for example in ``setup.py``.
-    ::
-
-        import numpy as np
-        ...
-        Extension('extension_name', ...
-                include_dirs=[np.get_numarray_include()])
-        ...
-
-    """
-    from numpy.numarray import get_numarray_include_dirs
-    include_dirs = get_numarray_include_dirs()
-    if type is None:
-        return include_dirs[0]
-    else:
-        return include_dirs + [get_include()]
-
 
 def _set_function_name(func, name):
     func.__name__ = name
     return func
 
 
-class _Deprecate(object):
+class _Deprecate:
     """
     Decorator class to deprecate old functions.
 
@@ -100,6 +61,7 @@ class _Deprecate(object):
     deprecate
 
     """
+
     def __init__(self, old_name=None, new_name=None, message=None):
         self.old_name = old_name
         self.new_name = new_name
@@ -114,7 +76,6 @@ class _Deprecate(object):
         new_name = self.new_name
         message = self.message
 
-        import warnings
         if old_name is None:
             try:
                 old_name = func.__name__
@@ -131,7 +92,7 @@ class _Deprecate(object):
 
         def newfunc(*args,**kwds):
             """`arrayrange` is deprecated, use `arange` instead!"""
-            warnings.warn(depdoc, DeprecationWarning)
+            warnings.warn(depdoc, DeprecationWarning, stacklevel=2)
             return func(*args, **kwds)
 
         newfunc = _set_function_name(newfunc, old_name)
@@ -139,6 +100,21 @@ class _Deprecate(object):
         if doc is None:
             doc = depdoc
         else:
+            lines = doc.expandtabs().split('\n')
+            indent = _get_indent(lines[1:])
+            if lines[0].lstrip():
+                # Indent the original first line to let inspect.cleandoc()
+                # dedent the docstring despite the deprecation notice.
+                doc = indent * ' ' + doc
+            else:
+                # Remove the same leading blank lines as cleandoc() would.
+                skip = len(lines[0]) + 1
+                for line in lines[1:]:
+                    if len(line) > indent:
+                        break
+                    skip += len(line) + 1
+                doc = doc[skip:]
+            depdoc = textwrap.indent(depdoc, ' ' * indent)
             doc = '\n\n'.join([depdoc, doc])
         newfunc.__doc__ = doc
         try:
@@ -149,6 +125,21 @@ class _Deprecate(object):
             newfunc.__dict__.update(d)
         return newfunc
 
+
+def _get_indent(lines):
+    """
+    Determines the leading whitespace that could be removed from all the lines.
+    """
+    indent = sys.maxsize
+    for line in lines:
+        content = len(line.lstrip())
+        if content:
+            indent = min(indent, len(line) - content)
+    if indent == sys.maxsize:
+        indent = 0
+    return indent
+
+
 def deprecate(*args, **kwargs):
     """
     Issues a DeprecationWarning, adds warning to `old_name`'s
@@ -162,16 +153,16 @@ def deprecate(*args, **kwargs):
     func : function
         The function to be deprecated.
     old_name : str, optional
-        The name of the function to be deprecated. Default is None, in which
-        case the name of `func` is used.
+        The name of the function to be deprecated. Default is None, in
+        which case the name of `func` is used.
     new_name : str, optional
-        The new name for the function. Default is None, in which case
-        the deprecation message is that `old_name` is deprecated. If given,
-        the deprecation message is that `old_name` is deprecated and `new_name`
+        The new name for the function. Default is None, in which case the
+        deprecation message is that `old_name` is deprecated. If given, the
+        deprecation message is that `old_name` is deprecated and `new_name`
         should be used instead.
     message : str, optional
-        Additional explanation of the deprecation.  Displayed in the docstring
-        after the warning.
+        Additional explanation of the deprecation.  Displayed in the
+        docstring after the warning.
 
     Returns
     -------
@@ -180,13 +171,12 @@ def deprecate(*args, **kwargs):
 
     Examples
     --------
-    Note that ``olduint`` returns a value after printing Deprecation Warning:
+    Note that ``olduint`` returns a value after printing Deprecation
+    Warning:
 
     >>> olduint = np.deprecate(np.uint)
+    DeprecationWarning: `uint64` is deprecated! # may vary
     >>> olduint(6)
-    /usr/lib/python2.5/site-packages/numpy/lib/utils.py:114:
-    DeprecationWarning: uint32 is deprecated
-      warnings.warn(str1, DeprecationWarning)
     6
 
     """
@@ -198,18 +188,36 @@ def deprecate(*args, **kwargs):
         fn = args[0]
         args = args[1:]
 
-        # backward compatibility -- can be removed
-        # after next release
-        if 'newname' in kwargs:
-            kwargs['new_name'] = kwargs.pop('newname')
-        if 'oldname' in kwargs:
-            kwargs['old_name'] = kwargs.pop('oldname')
-
         return _Deprecate(*args, **kwargs)(fn)
     else:
         return _Deprecate(*args, **kwargs)
 
-deprecate_with_doc = lambda msg: _Deprecate(message=msg)
+
+def deprecate_with_doc(msg):
+    """
+    Deprecates a function and includes the deprecation in its docstring.
+
+    This function is used as a decorator. It returns an object that can be
+    used to issue a DeprecationWarning, by passing the to-be decorated
+    function as argument, this adds warning to the to-be decorated function's
+    docstring and returns the new function object.
+
+    See Also
+    --------
+    deprecate : Decorate a function such that it issues a `DeprecationWarning`
+
+    Parameters
+    ----------
+    msg : str
+        Additional explanation of the deprecation. Displayed in the
+        docstring after the warning.
+
+    Returns
+    -------
+    obj : object
+
+    """
+    return _Deprecate(message=msg)
 
 
 #--------------------------------------------
@@ -223,14 +231,16 @@ def byte_bounds(a):
     Parameters
     ----------
     a : ndarray
-        Input array. It must conform to the Python-side of the array interface.
+        Input array. It must conform to the Python-side of the array
+        interface.
 
     Returns
     -------
     (low, high) : tuple of 2 integers
-        The first integer is the first byte of the array, the second integer is
-        just past the last byte of the array.  If `a` is not contiguous it
-        will not use every byte between the (`low`, `high`) values.
+        The first integer is the first byte of the array, the second
+        integer is just past the last byte of the array.  If `a` is not
+        contiguous it will not use every byte between the (`low`, `high`)
+        values.
 
     Examples
     --------
@@ -239,8 +249,8 @@ def byte_bounds(a):
     >>> low, high = np.byte_bounds(I)
     >>> high - low == I.size*I.itemsize
     True
-    >>> I = np.eye(2, dtype='G'); I.dtype
-    dtype('complex192')
+    >>> I = np.eye(2); I.dtype
+    dtype('float64')
     >>> low, high = np.byte_bounds(I)
     >>> high - low == I.size*I.itemsize
     True
@@ -250,11 +260,11 @@ def byte_bounds(a):
     a_data = ai['data'][0]
     astrides = ai['strides']
     ashape = ai['shape']
-   
     bytes_a = asarray(a).dtype.itemsize
-       
+
     a_low = a_high = a_data
-    if astrides is None: # contiguous case
+    if astrides is None:
+        # contiguous case
         a_high += a.size * bytes_a
     else:
         for shape, stride in zip(ashape, astrides):
@@ -273,10 +283,10 @@ def byte_bounds(a):
 
 def who(vardict=None):
     """
-    Print the Numpy arrays in the given dictionary.
+    Print the NumPy arrays in the given dictionary.
 
     If there is no dictionary passed in or `vardict` is None then returns
-    Numpy arrays in the globals() dictionary (all Numpy arrays in the
+    NumPy arrays in the globals() dictionary (all NumPy arrays in the
     namespace).
 
     Parameters
@@ -291,8 +301,8 @@ def who(vardict=None):
 
     Notes
     -----
-    Prints out the name, shape, bytes and type of all of the ndarrays present
-    in `vardict`.
+    Prints out the name, shape, bytes and type of all of the ndarrays
+    present in `vardict`.
 
     Examples
     --------
@@ -301,17 +311,17 @@ def who(vardict=None):
     >>> np.who()
     Name            Shape            Bytes            Type
     ===========================================================
-    a               10               40               int32
+    a               10               80               int64
     b               20               160              float64
-    Upper bound on total bytes  =       200
+    Upper bound on total bytes  =       240
 
     >>> d = {'x': np.arange(2.0), 'y': np.arange(3.0), 'txt': 'Some str',
     ... 'idx':5}
     >>> np.who(d)
     Name            Shape            Bytes            Type
     ===========================================================
-    y               3                24               float64
     x               2                16               float64
+    y               3                24               float64
     Upper bound on total bytes  =       40
 
     """
@@ -326,11 +336,11 @@ def who(vardict=None):
             idv = id(var)
             if idv in cache.keys():
                 namestr = name + " (%s)" % cache[idv]
-                original=0
+                original = 0
             else:
                 cache[idv] = name
                 namestr = name
-                original=1
+                original = 1
             shapestr = " x ".join(map(str, var.shape))
             bytestr = str(var.nbytes)
             sta.append([namestr, shapestr, bytestr, var.dtype.name,
@@ -340,8 +350,7 @@ def who(vardict=None):
     maxshape = 0
     maxbyte = 0
     totalbytes = 0
-    for k in range(len(sta)):
-        val = sta[k]
+    for val in sta:
         if maxname < len(val[0]):
             maxname = len(val[0])
         if maxshape < len(val[1]):
@@ -358,8 +367,7 @@ def who(vardict=None):
         prval = "Name %s Shape %s Bytes %s Type" % (sp1*' ', sp2*' ', sp3*' ')
         print(prval + "\n" + "="*(len(prval)+5) + "\n")
 
-    for k in range(len(sta)):
-        val = sta[k]
+    for val in sta:
         print("%s %s %s %s %s %s %s" % (val[0], ' '*(sp1-len(val[0])+4),
                                         val[1], ' '*(sp2-len(val[1])+5),
                                         val[2], ' '*(sp3-len(val[2])+5),
@@ -370,12 +378,12 @@ def who(vardict=None):
 #-----------------------------------------------------------------------------
 
 
-# NOTE:  pydoc defines a help function which works simliarly to this
+# NOTE:  pydoc defines a help function which works similarly to this
 #  except it uses a pager to take over the screen.
 
-# combine name and arguments and split to multiple lines of
-#  width characters.  End lines on a comma and begin argument list
-#  indented with the rest of the arguments.
+# combine name and arguments and split to multiple lines of width
+# characters.  End lines on a comma and begin argument list indented with
+# the rest of the arguments.
 def _split_line(name, arguments, width):
     firstwidth = len(name)
     k = firstwidth
@@ -419,7 +427,63 @@ def _makenamedict(module='numpy'):
                     thedict[modname] = moddict
     return thedict, dictlist
 
-def info(object=None,maxwidth=76,output=sys.stdout,toplevel='numpy'):
+
+def _info(obj, output=None):
+    """Provide information about ndarray obj.
+
+    Parameters
+    ----------
+    obj : ndarray
+        Must be ndarray, not checked.
+    output
+        Where printed output goes.
+
+    Notes
+    -----
+    Copied over from the numarray module prior to its removal.
+    Adapted somewhat as only numpy is an option now.
+
+    Called by info.
+
+    """
+    extra = ""
+    tic = ""
+    bp = lambda x: x
+    cls = getattr(obj, '__class__', type(obj))
+    nm = getattr(cls, '__name__', cls)
+    strides = obj.strides
+    endian = obj.dtype.byteorder
+
+    if output is None:
+        output = sys.stdout
+
+    print("class: ", nm, file=output)
+    print("shape: ", obj.shape, file=output)
+    print("strides: ", strides, file=output)
+    print("itemsize: ", obj.itemsize, file=output)
+    print("aligned: ", bp(obj.flags.aligned), file=output)
+    print("contiguous: ", bp(obj.flags.contiguous), file=output)
+    print("fortran: ", obj.flags.fortran, file=output)
+    print(
+        "data pointer: %s%s" % (hex(obj.ctypes._as_parameter_.value), extra),
+        file=output
+        )
+    print("byteorder: ", end=' ', file=output)
+    if endian in ['|', '=']:
+        print("%s%s%s" % (tic, sys.byteorder, tic), file=output)
+        byteswap = False
+    elif endian == '>':
+        print("%sbig%s" % (tic, tic), file=output)
+        byteswap = sys.byteorder != "big"
+    else:
+        print("%slittle%s" % (tic, tic), file=output)
+        byteswap = sys.byteorder != "little"
+    print("byteswap: ", bp(byteswap), file=output)
+    print("type: %s" % obj.dtype, file=output)
+
+
+@set_module('numpy')
+def info(object=None, maxwidth=76, output=None, toplevel='numpy'):
     """
     Get help information for a function, class, or module.
 
@@ -428,12 +492,13 @@ def info(object=None,maxwidth=76,output=sys.stdout,toplevel='numpy'):
     object : object or str, optional
         Input object or name to get information about. If `object` is a
         numpy object, its docstring is given. If it is a string, available
-        modules are searched for matching objects.
-        If None, information about `info` itself is returned.
+        modules are searched for matching objects.  If None, information
+        about `info` itself is returned.
     maxwidth : int, optional
         Printing width.
     output : file like object, optional
-        File like object that the output is written to, default is ``stdout``.
+        File like object that the output is written to, default is
+        ``None``, in which case ``sys.stdout`` will be used.
         The object has to be opened in 'w' or 'a' mode.
     toplevel : str, optional
         Start search at this level.
@@ -444,8 +509,9 @@ def info(object=None,maxwidth=76,output=sys.stdout,toplevel='numpy'):
 
     Notes
     -----
-    When used interactively with an object, ``np.info(obj)`` is equivalent to
-    ``help(obj)`` on the Python prompt or ``obj?`` on the IPython prompt.
+    When used interactively with an object, ``np.info(obj)`` is equivalent
+    to ``help(obj)`` on the Python prompt or ``obj?`` on the IPython
+    prompt.
 
     Examples
     --------
@@ -469,19 +535,22 @@ def info(object=None,maxwidth=76,output=sys.stdout,toplevel='numpy'):
     """
     global _namedict, _dictlist
     # Local import to speed up numpy's import time.
-    import pydoc, inspect
+    import pydoc
+    import inspect
 
-    if hasattr(object, '_ppimport_importer') or \
-       hasattr(object, '_ppimport_module'):
+    if (hasattr(object, '_ppimport_importer') or
+           hasattr(object, '_ppimport_module')):
         object = object._ppimport_module
     elif hasattr(object, '_ppimport_attr'):
         object = object._ppimport_attr
 
+    if output is None:
+        output = sys.stdout
+
     if object is None:
         info(info)
     elif isinstance(object, ndarray):
-        import numpy.numarray as nn
-        nn.info(object, output=output, numpy=1)
+        _info(object, output=output)
     elif isinstance(object, str):
         if _namedict is None:
             _namedict, _dictlist = _makenamedict(toplevel)
@@ -491,7 +560,10 @@ def info(object=None,maxwidth=76,output=sys.stdout,toplevel='numpy'):
             try:
                 obj = _namedict[namestr][object]
                 if id(obj) in objlist:
-                    print("\n     *** Repeat reference found in %s *** " % namestr, file=output)
+                    print("\n     "
+                          "*** Repeat reference found in %s *** " % namestr,
+                          file=output
+                          )
                 else:
                     objlist.append(id(obj))
                     print("     *** Found in %s ***" % namestr, file=output)
@@ -503,11 +575,17 @@ def info(object=None,maxwidth=76,output=sys.stdout,toplevel='numpy'):
         if numfound == 0:
             print("Help for %s not found." % object, file=output)
         else:
-            print("\n     *** Total of %d references found. ***" % numfound, file=output)
+            print("\n     "
+                  "*** Total of %d references found. ***" % numfound,
+                  file=output
+                  )
 
-    elif inspect.isfunction(object):
+    elif inspect.isfunction(object) or inspect.ismethod(object):
         name = object.__name__
-        arguments = inspect.formatargspec(*inspect.getargspec(object))
+        try:
+            arguments = str(inspect.signature(object))
+        except Exception:
+            arguments = "()"
 
         if len(name+arguments) > maxwidth:
             argstr = _split_line(name, arguments, maxwidth)
@@ -519,16 +597,10 @@ def info(object=None,maxwidth=76,output=sys.stdout,toplevel='numpy'):
 
     elif inspect.isclass(object):
         name = object.__name__
-        arguments = "()"
         try:
-            if hasattr(object, '__init__'):
-                arguments = inspect.formatargspec(*inspect.getargspec(object.__init__.__func__))
-                arglist = arguments.split(', ')
-                if len(arglist) > 1:
-                    arglist[1] = "("+arglist[1]
-                    arguments = ", ".join(arglist[1:])
-        except:
-            pass
+            arguments = str(inspect.signature(object))
+        except Exception:
+            arguments = "()"
 
         if len(name+arguments) > maxwidth:
             argstr = _split_line(name, arguments, maxwidth)
@@ -544,74 +616,26 @@ def info(object=None,maxwidth=76,output=sys.stdout,toplevel='numpy'):
             print(inspect.getdoc(object), file=output)
 
         methods = pydoc.allmethods(object)
-        if methods != []:
+
+        public_methods = [meth for meth in methods if meth[0] != '_']
+        if public_methods:
             print("\n\nMethods:\n", file=output)
-            for meth in methods:
-                if meth[0] == '_':
-                    continue
+            for meth in public_methods:
                 thisobj = getattr(object, meth, None)
                 if thisobj is not None:
-                    methstr, other = pydoc.splitdoc(inspect.getdoc(thisobj) or "None")
+                    methstr, other = pydoc.splitdoc(
+                            inspect.getdoc(thisobj) or "None"
+                            )
                 print("  %s  --  %s" % (meth, methstr), file=output)
 
-    elif (sys.version_info[0] < 3
-            and isinstance(object, types.InstanceType)):
-        # check for __call__ method
-        # types.InstanceType is the type of the instances of oldstyle classes
-        print("Instance of class: ", object.__class__.__name__, file=output)
-        print(file=output)
-        if hasattr(object, '__call__'):
-            arguments = inspect.formatargspec(*inspect.getargspec(object.__call__.__func__))
-            arglist = arguments.split(', ')
-            if len(arglist) > 1:
-                arglist[1] = "("+arglist[1]
-                arguments = ", ".join(arglist[1:])
-            else:
-                arguments = "()"
-
-            if hasattr(object, 'name'):
-                name = "%s" % object.name
-            else:
-                name = "<name>"
-            if len(name+arguments) > maxwidth:
-                argstr = _split_line(name, arguments, maxwidth)
-            else:
-                argstr = name + arguments
-
-            print(" " + argstr + "\n", file=output)
-            doc = inspect.getdoc(object.__call__)
-            if doc is not None:
-                print(inspect.getdoc(object.__call__), file=output)
-            print(inspect.getdoc(object), file=output)
-
-        else:
-            print(inspect.getdoc(object), file=output)
-
-    elif inspect.ismethod(object):
-        name = object.__name__
-        arguments = inspect.formatargspec(*inspect.getargspec(object.__func__))
-        arglist = arguments.split(', ')
-        if len(arglist) > 1:
-            arglist[1] = "("+arglist[1]
-            arguments = ", ".join(arglist[1:])
-        else:
-            arguments = "()"
-
-        if len(name+arguments) > maxwidth:
-            argstr = _split_line(name, arguments, maxwidth)
-        else:
-            argstr = name + arguments
-
-        print(" " + argstr + "\n", file=output)
-        print(inspect.getdoc(object), file=output)
-
     elif hasattr(object, '__doc__'):
         print(inspect.getdoc(object), file=output)
 
 
+@set_module('numpy')
 def source(object, output=sys.stdout):
     """
-    Print or write to a file the source code for a Numpy object.
+    Print or write to a file the source code for a NumPy object.
 
     The source code is only returned for objects written in Python. Many
     functions and classes are defined in C and will therefore not return
@@ -620,7 +644,8 @@ def source(object, output=sys.stdout):
     Parameters
     ----------
     object : numpy object
-        Input object. This can be any object (function, class, module, ...).
+        Input object. This can be any object (function, class, module,
+        ...).
     output : file object, optional
         If `output` not supplied then source code is printed to screen
         (sys.stdout).  File object must be created with either write 'w' or
@@ -652,7 +677,7 @@ def source(object, output=sys.stdout):
     try:
         print("In file: %s\n" % inspect.getsourcefile(object), file=output)
         print(inspect.getsource(object), file=output)
-    except:
+    except Exception:
         print("Not available for this object.", file=output)
 
 
@@ -661,15 +686,18 @@ def source(object, output=sys.stdout):
 # and index: index in breadth-first namespace traversal
 _lookfor_caches = {}
 
-# regexp whose match indicates that the string may contain a function signature
+# regexp whose match indicates that the string may contain a function
+# signature
 _function_signature_re = re.compile(r"[a-z0-9_]+\(.*[,=].*\)", re.I)
 
+
+@set_module('numpy')
 def lookfor(what, module=None, import_modules=True, regenerate=False,
             output=None):
     """
     Do a keyword search on docstrings.
 
-    A list of of objects that matched the search is displayed,
+    A list of objects that matched the search is displayed,
     sorted by relevance. All given keywords need to be found in the
     docstring for it to be returned as a result, but the order does
     not matter.
@@ -698,7 +726,7 @@ def lookfor(what, module=None, import_modules=True, regenerate=False,
 
     Examples
     --------
-    >>> np.lookfor('binary representation')
+    >>> np.lookfor('binary representation') # doctest: +SKIP
     Search results for 'binary representation'
     ------------------------------------------
     numpy.binary_repr
@@ -719,19 +747,15 @@ def lookfor(what, module=None, import_modules=True, regenerate=False,
     # XXX: maybe using a real stemming search engine would be better?
     found = []
     whats = str(what).lower().split()
-    if not whats: return
+    if not whats:
+        return
 
     for name, (docstring, kind, index) in cache.items():
         if kind in ('module', 'object'):
             # don't show modules or objects
             continue
-        ok = True
         doc = docstring.lower()
-        for w in whats:
-            if w not in doc:
-                ok = False
-                break
-        if ok:
+        if all(w in doc for w in whats):
             found.append(name)
 
     # Relevance sort
@@ -811,15 +835,10 @@ def _lookfor_generate_cache(module, import_modules, regenerate):
         or newly generated.
 
     """
-    global _lookfor_caches
     # Local import to speed up numpy's import time.
     import inspect
 
-    if sys.version_info[0] >= 3:
-        # In Python3 stderr, stdout are text files.
-        from io import StringIO
-    else:
-        from StringIO import StringIO
+    from io import StringIO
 
     if module is None:
         module = "numpy"
@@ -848,7 +867,8 @@ def _lookfor_generate_cache(module, import_modules, regenerate):
     stack = [(module.__name__, module)]
     while stack:
         name, item = stack.pop(0)
-        if id(item) in seen: continue
+        if id(item) in seen:
+            continue
         seen[id(item)] = True
 
         index += 1
@@ -867,7 +887,8 @@ def _lookfor_generate_cache(module, import_modules, regenerate):
                     for mod_path in os.listdir(pth):
                         this_py = os.path.join(pth, mod_path)
                         init_py = os.path.join(pth, mod_path, '__init__.py')
-                        if os.path.isfile(this_py) and mod_path.endswith('.py'):
+                        if (os.path.isfile(this_py) and
+                                mod_path.endswith('.py')):
                             to_import = mod_path[:-3]
                         elif os.path.isfile(init_py):
                             to_import = mod_path
@@ -876,13 +897,6 @@ def _lookfor_generate_cache(module, import_modules, regenerate):
                         if to_import == '__init__':
                             continue
 
-                        try:
-                            # Catch SystemExit, too
-                            base_exc = BaseException
-                        except NameError:
-                            # Python 2.4 doesn't have BaseException
-                            base_exc = Exception
-
                         try:
                             old_stdout = sys.stdout
                             old_stderr = sys.stderr
@@ -893,7 +907,8 @@ def _lookfor_generate_cache(module, import_modules, regenerate):
                             finally:
                                 sys.stdout = old_stdout
                                 sys.stderr = old_stderr
-                        except base_exc:
+                        # Catch SystemExit, too
+                        except (Exception, SystemExit):
                             continue
 
             for n, v in _getmembers(item):
@@ -927,7 +942,8 @@ def _lookfor_generate_cache(module, import_modules, regenerate):
 
         try:
             doc = inspect.getdoc(item)
-        except NameError: # ref SWIG's NameError: Unknown C global variable
+        except NameError:
+            # ref SWIG's NameError: Unknown C global variable
             doc = None
         if doc is not None:
             cache[name] = (doc, kind, index)
@@ -938,136 +954,11 @@ def _getmembers(item):
     import inspect
     try:
         members = inspect.getmembers(item)
-    except AttributeError:
+    except Exception:
         members = [(x, getattr(item, x)) for x in dir(item)
                    if hasattr(item, x)]
     return members
 
-#-----------------------------------------------------------------------------
-
-# The following SafeEval class and company are adapted from Michael Spencer's
-# ASPN Python Cookbook recipe:
-#   http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/364469
-# Accordingly it is mostly Copyright 2006 by Michael Spencer.
-# The recipe, like most of the other ASPN Python Cookbook recipes was made
-# available under the Python license.
-#   http://www.python.org/license
-
-# It has been modified to:
-#   * handle unary -/+
-#   * support True/False/None
-#   * raise SyntaxError instead of a custom exception.
-
-class SafeEval(object):
-    """
-    Object to evaluate constant string expressions.
-
-    This includes strings with lists, dicts and tuples using the abstract
-    syntax tree created by ``compiler.parse``.
-
-    For an example of usage, see `safe_eval`.
-
-    See Also
-    --------
-    safe_eval
-
-    """
-
-    if sys.version_info[0] < 3:
-        def visit(self, node, **kw):
-            cls = node.__class__
-            meth = getattr(self, 'visit'+cls.__name__, self.default)
-            return meth(node, **kw)
-
-        def default(self, node, **kw):
-            raise SyntaxError("Unsupported source construct: %s"
-                              % node.__class__)
-
-        def visitExpression(self, node, **kw):
-            for child in node.getChildNodes():
-                return self.visit(child, **kw)
-
-        def visitConst(self, node, **kw):
-            return node.value
-
-        def visitDict(self, node,**kw):
-            return dict([(self.visit(k), self.visit(v)) for k, v in node.items])
-
-        def visitTuple(self, node, **kw):
-            return tuple([self.visit(i) for i in node.nodes])
-
-        def visitList(self, node, **kw):
-            return [self.visit(i) for i in node.nodes]
-
-        def visitUnaryAdd(self, node, **kw):
-            return +self.visit(node.getChildNodes()[0])
-
-        def visitUnarySub(self, node, **kw):
-            return -self.visit(node.getChildNodes()[0])
-
-        def visitName(self, node, **kw):
-            if node.name == 'False':
-                return False
-            elif node.name == 'True':
-                return True
-            elif node.name == 'None':
-                return None
-            else:
-                raise SyntaxError("Unknown name: %s" % node.name)
-    else:
-
-        def visit(self, node):
-            cls = node.__class__
-            meth = getattr(self, 'visit' + cls.__name__, self.default)
-            return meth(node)
-
-        def default(self, node):
-            raise SyntaxError("Unsupported source construct: %s"
-                              % node.__class__)
-
-        def visitExpression(self, node):
-            return self.visit(node.body)
-
-        def visitNum(self, node):
-            return node.n
-
-        def visitStr(self, node):
-            return node.s
-
-        def visitBytes(self, node):
-            return node.s
-
-        def visitDict(self, node,**kw):
-            return dict([(self.visit(k), self.visit(v))
-                         for k, v in zip(node.keys, node.values)])
-
-        def visitTuple(self, node):
-            return tuple([self.visit(i) for i in node.elts])
-
-        def visitList(self, node):
-            return [self.visit(i) for i in node.elts]
-
-        def visitUnaryOp(self, node):
-            import ast
-            if isinstance(node.op, ast.UAdd):
-                return +self.visit(node.operand)
-            elif isinstance(node.op, ast.USub):
-                return -self.visit(node.operand)
-            else:
-                raise SyntaxError("Unknown unary op: %r" % node.op)
-
-        def visitName(self, node):
-            if node.id == 'False':
-                return False
-            elif node.id == 'True':
-                return True
-            elif node.id == 'None':
-                return None
-            else:
-                raise SyntaxError("Unknown name: %s" % node.id)
-
-        def visitNameConstant(self, node):
-            return node.value
 
 def safe_eval(source):
     """
@@ -1089,8 +980,8 @@ def safe_eval(source):
     Raises
     ------
     SyntaxError
-        If the code has invalid Python syntax, or if it contains non-literal
-        code.
+        If the code has invalid Python syntax, or if it contains
+        non-literal code.
 
     Examples
     --------
@@ -1109,28 +1000,74 @@ def safe_eval(source):
     >>> np.safe_eval('open("/home/user/.ssh/id_dsa").read()')
     Traceback (most recent call last):
       ...
-    SyntaxError: Unsupported source construct: compiler.ast.CallFunc
+    ValueError: malformed node or string: <_ast.Call object at 0x...>
 
     """
-    # Local imports to speed up numpy's import time.
-    import warnings
+    # Local import to speed up numpy's import time.
+    import ast
+    return ast.literal_eval(source)
 
-    with warnings.catch_warnings():
-        # compiler package is deprecated for 3.x, which is already solved here
-        warnings.simplefilter('ignore', DeprecationWarning)
-        try:
-            import compiler
-        except ImportError:
-            import ast as compiler
 
-    walker = SafeEval()
-    try:
-        ast = compiler.parse(source, mode="eval")
-    except SyntaxError as err:
-        raise
-    try:
-        return walker.visit(ast)
-    except SyntaxError as err:
-        raise
+def _median_nancheck(data, result, axis):
+    """
+    Utility function to check median result from data for NaN values at the end
+    and return NaN in that case. Input result can also be a MaskedArray.
 
+    Parameters
+    ----------
+    data : array
+        Sorted input data to median function
+    result : Array or MaskedArray
+        Result of median function.
+    axis : int
+        Axis along which the median was computed.
+
+    Returns
+    -------
+    result : scalar or ndarray
+        Median or NaN in axes which contained NaN in the input.  If the input
+        was an array, NaN will be inserted in-place.  If a scalar, either the
+        input itself or a scalar NaN.
+    """
+    if data.size == 0:
+        return result
+    n = np.isnan(data.take(-1, axis=axis))
+    # masked NaN values are ok
+    if np.ma.isMaskedArray(n):
+        n = n.filled(False)
+    if np.count_nonzero(n.ravel()) > 0:
+        # Without given output, it is possible that the current result is a
+        # numpy scalar, which is not writeable.  If so, just return nan.
+        if isinstance(result, np.generic):
+            return data.dtype.type(np.nan)
+
+        result[n] = np.nan
+    return result
+
+def _opt_info():
+    """
+    Returns a string contains the supported CPU features by the current build.
+
+    The string format can be explained as follows:
+        - dispatched features that are supported by the running machine
+          end with `*`.
+        - dispatched features that are "not" supported by the running machine
+          end with `?`.
+        - remained features are representing the baseline.
+    """
+    from numpy.core._multiarray_umath import (
+        __cpu_features__, __cpu_baseline__, __cpu_dispatch__
+    )
+
+    if len(__cpu_baseline__) == 0 and len(__cpu_dispatch__) == 0:
+        return ''
+
+    enabled_features = ' '.join(__cpu_baseline__)
+    for feature in __cpu_dispatch__:
+        if __cpu_features__[feature]:
+            enabled_features += f" {feature}*"
+        else:
+            enabled_features += f" {feature}?"
+
+    return enabled_features
 #-----------------------------------------------------------------------------
